@@ -1,4 +1,3 @@
-import dataclasses
 from functools import cached_property
 from typing import (
     TYPE_CHECKING,
@@ -14,10 +13,10 @@ from typing import (
     TypeVar,
     Union,
     cast,
+    get_args,
     overload,
 )
 
-from django.core.exceptions import FieldDoesNotExist
 from django.db import models
 from django.db.models import QuerySet
 from django.db.models.fields.related_descriptors import (
@@ -25,43 +24,42 @@ from django.db.models.fields.related_descriptors import (
     ReverseManyToOneDescriptor,
     ReverseOneToOneDescriptor,
 )
-from django.db.models.fields.reverse_related import ManyToManyRel, ManyToOneRel
 from django.db.models.query_utils import DeferredAttribute
 import strawberry
 from strawberry.annotation import StrawberryAnnotation
-from strawberry.arguments import UNSET, StrawberryArgument, is_unset
+from strawberry.arguments import UNSET, StrawberryArgument
+from strawberry.lazy_type import LazyType
 from strawberry.permission import BasePermission
 from strawberry.schema_directive import StrawberrySchemaDirective
+from strawberry.type import StrawberryContainer, StrawberryType
 from strawberry.types.fields.resolver import StrawberryResolver
 from strawberry.types.info import Info
+from strawberry.types.types import TypeDefinition
 from strawberry.union import StrawberryUnion
 from strawberry_django.arguments import argument
 from strawberry_django.fields.field import (
     StrawberryDjangoField as _StrawberryDjangoField,
 )
-from strawberry_django.fields.types import (
-    get_model_field,
-    is_auto,
-    is_optional,
-    resolve_model_field_name,
-)
-from strawberry_django.utils import is_similar_django_type, unwrap_type
-from typing_extensions import Self
+from strawberry_django.fields.types import is_optional
+from strawberry_django.utils import get_django_model, unwrap_type
 
 from . import relay
 from .descriptors import ModelProperty
 from .optimizer import OptimizerStore, PrefetchType
 from .permissions import filter_with_perms
-from .types import resolve_model_field_type
 from .utils import resolvers
 from .utils.typing import TypeOrSequence
 
 if TYPE_CHECKING:
-    from .type import StrawberryDjangoType
+    pass
 
 __all__ = [
     "StrawberryDjangoField",
+    "StrawberryDjangoNodeField",
+    "StrawberryDjangoConnectionField",
     "field",
+    "node",
+    "connection",
 ]
 
 _T = TypeVar("_T")
@@ -87,6 +85,10 @@ class StrawberryDjangoField(_StrawberryDjangoField):
 
     @property
     def arguments(self) -> List[StrawberryArgument]:
+        if isinstance(self, relay.NodeField):
+            return []
+
+        args = super().arguments
         is_node = isinstance(unwrap_type(self.type), relay.Node)
         return [
             (
@@ -100,15 +102,51 @@ class StrawberryDjangoField(_StrawberryDjangoField):
                 and arg.type_annotation.annotation == strawberry.ID
                 else arg
             )
-            for arg in super().arguments
+            for arg in args
         ]
 
+    @property
+    def type(self) -> Union[StrawberryType, type]:  # noqa:A003
+        return super().type
+
+    @type.setter
+    def type(self, type_: Any) -> None:  # noqa:A003
+        if type_ is not None and self.origin_django_type is None:
+            resolved = type_
+            if isinstance(resolved, StrawberryAnnotation):
+                resolved = type_.resolve()
+            while isinstance(resolved, StrawberryContainer):
+                resolved = resolved.of_type
+
+            dj_type = getattr(resolved, "_django_type", None)
+            if dj_type is None:
+                contained = get_args(resolved)
+                if contained:
+                    dj_type = getattr(contained[0], "_django_type", None)
+
+            if dj_type is not None:
+                self.origin_django_type = dj_type
+                self.filters = dj_type.filters
+                self.order = dj_type.order
+                self.pagination = dj_type.pagination
+
+        self.type_annotation = type_
+
     @cached_property
     def model(self) -> Type[models.Model]:
-        model = self.django_model
+        type_ = unwrap_type(self.type)
+        model = get_django_model(type_)
         if model:
             return model
 
+        tdef = cast(Optional[TypeDefinition], getattr(type_, "_type_definition", None))
+        if tdef and tdef.concrete_of and issubclass(tdef.concrete_of.origin, relay.Connection):
+            n_type = tdef.type_var_map[relay.NodeType]  # type:ignore
+            if isinstance(n_type, LazyType):
+                n_type = n_type.resolve_type()
+
+            return cast(Type[models.Model], get_django_model(n_type))
+
         origin = self.origin_django_type or getattr(self.origin, "_django_type", None)
         model = origin and origin.model
 
@@ -135,126 +173,6 @@ class StrawberryDjangoField(_StrawberryDjangoField):
             resolver = resolvers.async_safe(resolver)
         return resolver
 
-    @classmethod
-    def from_django_type(
-        cls,
-        django_type: "StrawberryDjangoType",
-        name: str,
-        *,
-        type_annotation: Optional[StrawberryAnnotation] = None,
-    ) -> Self:
-        origin = django_type.origin
-
-        attr = getattr(origin, name, UNSET)
-        if is_unset(attr):
-            attr = getattr(cls, "__dataclass_fields__", {}).get(name, UNSET)
-        if attr is dataclasses.MISSING:
-            attr = UNSET
-
-        if isinstance(attr, cls) and not attr.origin_django_type:
-            field = cast(Self, attr)
-        elif isinstance(attr, dataclasses.Field):
-            default = getattr(attr, "default", UNSET)
-            if default is dataclasses.MISSING:
-                default = UNSET
-
-            default_factory = getattr(attr, "default_factory", UNSET)
-            if default_factory is dataclasses.MISSING:
-                default_factory = UNSET
-
-            if type_annotation is None:
-                type_annotation = getattr(attr, "type_annotation", None)
-            if type_annotation is None:
-                type_annotation = StrawberryAnnotation(attr.type)
-
-            field = cls(
-                django_name=getattr(attr, "django_name", attr.name),
-                graphql_name=getattr(attr, "graphql_name", None),
-                origin=getattr(attr, "origin", None),
-                is_subscription=getattr(attr, "is_subscription", False),
-                description=getattr(attr, "description", None),
-                base_resolver=getattr(attr, "base_resolver", None),
-                permission_classes=getattr(attr, "permission_classes", ()),
-                default=default,
-                default_factory=default_factory,
-                deprecation_reason=getattr(attr, "deprecation_reason", None),
-                directives=getattr(attr, "directives", ()),
-                type_annotation=type_annotation,
-            )
-        elif isinstance(attr, StrawberryResolver):
-            field = cls(base_resolver=attr)
-        elif callable(attr):
-            field = cast(Self, cls()(attr))
-        else:
-            field = cls(default=attr)
-
-        field.python_name = name
-        # store origin django type for further usage
-        if name in origin.__dict__.get("__annotations__", {}):
-            field.origin_django_type = django_type
-
-        # annotation of field is used as a class type
-        if type_annotation is not None:
-            field.type_annotation = type_annotation
-            field.is_auto = is_auto(field.type_annotation)
-
-        # resolve the django_name and check if it is relation field. django_name
-        # is used to access the field data in resolvers
-        try:
-            model_field = get_model_field(django_type.model, field.django_name or name)
-        except FieldDoesNotExist:
-            model_attr = getattr(django_type.model, name, None)
-            if model_attr is not None and isinstance(model_attr, ModelProperty):
-                if field.is_auto:
-                    field.type_annotation = StrawberryAnnotation(model_attr.type_annotation)
-                    field.is_auto = is_auto(field.type_annotation)
-
-                if field.description is None:
-                    field.description = model_attr.description
-            elif field.django_name or field.is_auto:
-                raise  # field should exist, reraise caught exception
-        else:
-            field.is_relation = model_field.is_relation
-            field.django_name = resolve_model_field_name(
-                model_field,
-                is_input=django_type.is_input,
-                is_filter=django_type.is_filter,
-            )
-
-            # change relation field type to auto if field is inherited from another
-            # type. for example if field is inherited from output type but we are
-            # configuring field for input type
-            if field.is_relation and not is_similar_django_type(
-                django_type, field.origin_django_type
-            ):
-                field.is_auto = True
-
-            # resolve type of auto field
-            if field.is_auto:
-                field.type_annotation = StrawberryAnnotation(
-                    resolve_model_field_type(
-                        model_field,
-                        django_type,
-                        is_input=django_type.is_input,
-                        is_partial=django_type.is_partial,
-                    )
-                )
-
-            if field.description is None:
-                description = (
-                    model_field.field.help_text
-                    if isinstance(model_field, (ManyToOneRel, ManyToManyRel))
-                    else model_field.help_text
-                )
-                if description:
-                    field.description = str(description)
-
-        # FIXME: How to properly workaround this for mutations?
-        if django_type.is_input and is_unset(field.default_value):
-            field.default = UNSET
-
-        return field
-
     def get_result(
         self,
         source: Optional[models.Model],
@@ -312,7 +230,7 @@ class StrawberryDjangoField(_StrawberryDjangoField):
         return self.safe_resolver(*args, **kwargs)
 
     @resolvers.async_safe
-    def get_queryset_as_list(self, qs: QuerySet[_M], info: Info, **kwargs) -> List[_M]:
+    def get_queryset_as_list(self, qs: QuerySet[_M], info: Info, **kwargs) -> QuerySet[_M]:
         if not self.base_resolver:
             nodes: Optional[List[relay.GlobalID]] = kwargs.get("ids")
             if isinstance(nodes, list):
@@ -320,7 +238,12 @@ class StrawberryDjangoField(_StrawberryDjangoField):
                     assert {n.resolve_type(info) for n in nodes} == {unwrap_type(self.type)}
                 qs = qs.filter(pk__in=[n.node_id for n in nodes])
 
-        return list(self.get_queryset(filter_with_perms(qs, info), info, **kwargs))
+        qs = self.get_queryset(filter_with_perms(qs, info), info, **kwargs)
+        if not isinstance(self, relay.ConnectionField):
+            # This is what QuerySet does internally to fetch results.
+            # After this, iterating over the queryset should be async safe
+            qs._fetch_all()  # type:ignore
+        return qs
 
     @resolvers.async_safe
     def get_queryset_one(self, qs: QuerySet[_M], info: Info, **kwargs) -> Optional[_M]:
@@ -339,13 +262,31 @@ class StrawberryDjangoField(_StrawberryDjangoField):
         return None
 
 
+class StrawberryDjangoNodeField(relay.NodeField, StrawberryDjangoField):
+    ...
+
+
+class StrawberryDjangoConnectionField(relay.ConnectionField, StrawberryDjangoField):
+    def resolve_nodes(
+        self,
+        source: Any,
+        info: Info,
+        args: List[Any],
+        kwargs: Dict[str, Any],
+    ):
+        return resolvers.resolve_result(
+            self.model._default_manager.all(),
+            info=info,
+            qs_resolver=lambda qs: self.get_queryset_as_list(qs, info, **kwargs),
+        )
+
+
 @overload
 def field(
     *,
     resolver: Callable[[], _T],
     name: Optional[str] = None,
     field_name: Optional[str] = None,
-    filters: Any = UNSET,
     is_subscription: bool = False,
     description: Optional[str] = None,
     init: Literal[False] = False,
@@ -354,6 +295,9 @@ def field(
     default: Any = UNSET,
     default_factory: Union[Callable, object] = UNSET,
     directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    pagination: Optional[bool] = UNSET,
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
     only: Optional[TypeOrSequence[str]] = None,
     select_related: Optional[TypeOrSequence[str]] = None,
     prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
@@ -366,7 +310,6 @@ def field(
     *,
     name: Optional[str] = None,
     field_name: Optional[str] = None,
-    filters: Any = UNSET,
     is_subscription: bool = False,
     description: Optional[str] = None,
     init: Literal[True] = True,
@@ -375,6 +318,9 @@ def field(
     default: Any = UNSET,
     default_factory: Union[Callable, object] = UNSET,
     directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    pagination: Optional[bool] = UNSET,
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
     only: Optional[TypeOrSequence[str]] = None,
     select_related: Optional[TypeOrSequence[str]] = None,
     prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
@@ -388,7 +334,6 @@ def field(
     *,
     name: Optional[str] = None,
     field_name: Optional[str] = None,
-    filters: Any = UNSET,
     is_subscription: bool = False,
     description: Optional[str] = None,
     permission_classes: Optional[List[Type[BasePermission]]] = None,
@@ -396,6 +341,9 @@ def field(
     default: Any = UNSET,
     default_factory: Union[Callable, object] = UNSET,
     directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    pagination: Optional[bool] = UNSET,
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
     only: Optional[TypeOrSequence[str]] = None,
     select_related: Optional[TypeOrSequence[str]] = None,
     prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
@@ -408,7 +356,6 @@ def field(
     *,
     name: Optional[str] = None,
     field_name: Optional[str] = None,
-    filters: Any = UNSET,
     is_subscription: bool = False,
     description: Optional[str] = None,
     permission_classes: Optional[List[Type[BasePermission]]] = None,
@@ -416,6 +363,9 @@ def field(
     default: Any = UNSET,
     default_factory: Union[Callable, object] = UNSET,
     directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    pagination: Optional[bool] = UNSET,
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
     only: Optional[TypeOrSequence[str]] = None,
     select_related: Optional[TypeOrSequence[str]] = None,
     prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
@@ -451,6 +401,8 @@ def field(
         default_factory=default_factory,
         directives=directives,
         filters=filters,
+        pagination=pagination,
+        order=order,
         only=only,
         select_related=select_related,
         prefetch_related=prefetch_related,
@@ -458,3 +410,216 @@ def field(
     if resolver:
         f = f(resolver)
     return f
+
+
+def node(
+    *,
+    name: Optional[str] = None,
+    is_subscription: bool = False,
+    description: Optional[str] = None,
+    permission_classes: Optional[List[Type[BasePermission]]] = None,
+    deprecation_reason: Optional[str] = None,
+    default: Any = UNSET,
+    default_factory: Union[Callable, object] = UNSET,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+    # This init parameter is used by pyright to determine whether this field
+    # is added in the constructor or not. It is not used to change
+    # any behavior at the moment.
+    init=None,
+) -> Any:
+    """Annotate a property to create a relay query field.
+
+    Examples:
+        Annotating something like this:
+
+        >>> @strawberry.type
+        >>> class X:
+        ...     some_node: SomeType = relay.node(description="ABC")
+
+        Will produce a query like this that returns `SomeType` given its id.
+
+        ```
+        query {
+            someNode (id: ID) {
+                id
+                ...
+            }
+        }
+        ```
+
+    """
+    return StrawberryDjangoNodeField(
+        python_name=None,
+        graphql_name=name,
+        type_annotation=None,
+        description=description,
+        is_subscription=is_subscription,
+        permission_classes=permission_classes or [],
+        deprecation_reason=deprecation_reason,
+        default=default,
+        default_factory=default_factory,
+        directives=directives or (),
+        only=only,
+        select_related=select_related,
+        prefetch_related=prefetch_related,
+    )
+
+
+@overload
+def connection(
+    *,
+    resolver: Callable[[], _T],
+    name: Optional[str] = None,
+    is_subscription: bool = False,
+    description: Optional[str] = None,
+    init: Literal[False] = False,
+    permission_classes: Optional[List[Type[BasePermission]]] = None,
+    deprecation_reason: Optional[str] = None,
+    default: Any = UNSET,
+    default_factory: Union[Callable, object] = UNSET,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+) -> _T:
+    ...
+
+
+@overload
+def connection(
+    *,
+    name: Optional[str] = None,
+    is_subscription: bool = False,
+    description: Optional[str] = None,
+    init: Literal[True] = True,
+    permission_classes: Optional[List[Type[BasePermission]]] = None,
+    deprecation_reason: Optional[str] = None,
+    default: Any = UNSET,
+    default_factory: Union[Callable, object] = UNSET,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+) -> Any:
+    ...
+
+
+@overload
+def connection(
+    resolver: Union[StrawberryResolver, Callable, staticmethod, classmethod],
+    *,
+    name: Optional[str] = None,
+    is_subscription: bool = False,
+    description: Optional[str] = None,
+    permission_classes: Optional[List[Type[BasePermission]]] = None,
+    deprecation_reason: Optional[str] = None,
+    default: Any = UNSET,
+    default_factory: Union[Callable, object] = UNSET,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+) -> StrawberryDjangoConnectionField:
+    ...
+
+
+def connection(
+    resolver=None,
+    *,
+    name: Optional[str] = None,
+    is_subscription: bool = False,
+    description: Optional[str] = None,
+    permission_classes: Optional[List[Type[BasePermission]]] = None,
+    deprecation_reason: Optional[str] = None,
+    default: Any = UNSET,
+    default_factory: Union[Callable, object] = UNSET,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    filters: Optional[type] = UNSET,
+    order: Optional[type] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+    # This init parameter is used by pyright to determine whether this field
+    # is added in the constructor or not. It is not used to change
+    # any behavior at the moment.
+    init=None,
+) -> Any:
+    """Annotate a property or a method to create a relay connection field.
+
+    Relay connections_ are mostly used for pagination purposes. This decorator
+    helps creating a complete relay endpoint that provides default arguments
+    and has a default implementation for the connection slicing.
+
+    Note that when setting a resolver to this field, it is expected for this
+    resolver to return an iterable of the expected node type, not the connection
+    itself. That iterable will then be paginated accordingly. So, the main use
+    case for this is to provide a filtered iterable of nodes by using some custom
+    filter arguments.
+
+    Examples:
+        Annotating something like this:
+
+        >>> @strawberry.type
+        >>> class X:
+        ...     some_node: relay.Connection[SomeType] = relay.connection(description="ABC")
+        ...
+        ...     @relay.connection(description="ABC")
+        ...     def get_some_nodes(self, age: int) -> Iterable[SomeType]:
+        ...         ...
+
+        Will produce a query like this:
+
+        ```
+        query {
+            someNode (before: String, after: String, first: String, after: String, age: Int) {
+                totalCount
+                pageInfo {
+                    hasNextPage
+                    hasPreviousPage
+                    startCursor
+                    endCursor
+                }
+                edges {
+                    cursor
+                    node {
+                        id
+                        ...
+                    }
+                }
+            }
+        }
+        ```
+
+    .. _Relay connections:
+        https://relay.dev/graphql/connections.htm
+
+    """
+    f = StrawberryDjangoConnectionField(
+        python_name=None,
+        graphql_name=name,
+        type_annotation=None,
+        description=description,
+        is_subscription=is_subscription,
+        permission_classes=permission_classes or [],
+        deprecation_reason=deprecation_reason,
+        default=default,
+        default_factory=default_factory,
+        directives=directives or (),
+        filters=filters,
+        order=order,
+        only=only,
+        select_related=select_related,
+        prefetch_related=prefetch_related,
+    )
+    if resolver is not None:
+        f = f(resolver)
+    return f
