@@ -31,10 +31,8 @@ from strawberry.types.info import Info
 from strawberry.utils.await_maybe import AwaitableOrValue
 from typing_extensions import Self, final
 
-from strawberry_django_plus import relay
-
 from .directives import SchemaDirectiveHelper, SchemaDirectiveResolver, schema_directive
-from .relay import Connection
+from .relay import Connection, GlobalID
 from .types import OperationInfo, OperationMessage
 from .utils import aio, resolvers
 from .utils.query import filter_for_user
@@ -164,7 +162,7 @@ def get_with_perms(
 
 @overload
 def get_with_perms(
-    pk: relay.GlobalID,
+    pk: GlobalID,
     info: Info,
     *,
     required: Literal[True],
@@ -175,7 +173,7 @@ def get_with_perms(
 
 @overload
 def get_with_perms(
-    pk: relay.GlobalID,
+    pk: GlobalID,
     info: Info,
     *,
     required: bool = ...,
@@ -185,7 +183,7 @@ def get_with_perms(
 
 
 def get_with_perms(pk, info, *, required=False, model=None):
-    if isinstance(pk, relay.GlobalID):
+    if isinstance(pk, GlobalID):
         instance = pk.resolve_node(info, required=required, ensure_type=model)
         if aio.is_awaitable(instance, info=info):
             instance = resolvers.resolve_sync(instance)
@@ -488,7 +486,11 @@ class HasPermDirective(AuthDirective):
     obj_perm_checker: Private[
         Callable[[GraphQLResolveInfo, UserType], Callable[[PermDefinition, Any], bool]]
     ] = dataclasses.field(
-        default=lambda info, user: functools.partial(user.has_perm),
+        default=lambda info, user: lambda perm, obj: (
+            # Check global perms first, then object specific
+            user.has_perm(perm.perm)
+            or user.has_perm(perm.perm, obj=obj)
+        ),
     )
     with_anonymous: Private[bool] = dataclasses.field(default=True)
     with_superuser: Private[bool] = dataclasses.field(default=False)
@@ -632,7 +634,7 @@ class HasPermDirective(AuthDirective):
 
         f = any if self.any else all
         checker = self.obj_perm_checker(info, user)
-        has_perm = f(checker(p, root) for p in self.perms)
+        has_perm = f(checker(p, obj) for p in self.perms)
 
         cache[key] = has_perm
         return has_perm
