@@ -18,6 +18,7 @@ from django.db.models.fields import NOT_PROVIDED
 from django.db.models.fields.related import ManyToManyField
 from django.db.models.fields.reverse_related import ForeignObjectRel
 import strawberry
+from strawberry.arguments import UNSET
 from strawberry.custom_scalar import ScalarWrapper
 from strawberry.file_uploads import Upload
 from strawberry.type import StrawberryType
@@ -95,16 +96,10 @@ def register(
 
 
 @strawberry.type(
-    description="Input of an object that implements the `Node` interface.",
+    description="Generic type for objects that implements the `Node` interface.",
 )
 class NodeType(relay.Node):
-    """Set the value to the selected node.
-
-    Notes:
-        This can be used as a base class for input types that receive an
-        `id` of type `GlobalID` when inheriting from it.
-
-    """
+    """Set the value to the selected node."""
 
     id: relay.GlobalID  # noqa:A003
 
@@ -136,23 +131,23 @@ class NodeInputPartial(NodeInput):
 
     """
 
-    id: Optional[relay.GlobalID] = strawberry.field(default=None)  # noqa:A003
+    id: Optional[relay.GlobalID] = UNSET  # noqa:A003
 
 
 @strawberry.input(description=("Add/remove/set the selected nodes."))
 class ListInput(Generic[K]):
     """Add/remove/set the selected nodes."""
 
-    set: Optional[List[K]] = strawberry.field(default=None)  # noqa:A003
-    add: Optional[List[K]] = strawberry.field(default=None)
-    remove: Optional[List[K]] = strawberry.field(default=None)
+    set: Optional[List[K]] = UNSET  # noqa:A003
+    add: Optional[List[K]] = UNSET
+    remove: Optional[List[K]] = UNSET
 
 
 @strawberry.input(description=("Add/remove/set the selected nodes, passing `data` through."))
 class ListThroughInput(ListInput[K], Generic[K, D]):
     """Add/remove/set the selected nodes."""
 
-    data: Optional[D] = strawberry.field(default=None)
+    data: Optional[D] = UNSET
 
 
 @strawberry.type
@@ -192,9 +187,6 @@ class OperationInfo:
 def resolve_model_field_type(
     field: Union[models.Field, ForeignObjectRel],
     django_type: "StrawberryDjangoType",
-    *,
-    is_input: bool = False,
-    is_partial: bool = False,
 ):
     """Resolve type for model field."""
     if has_choices_field and isinstance(field, (TextChoicesField, IntegerChoicesField)):
@@ -229,16 +221,20 @@ def resolve_model_field_type(
             if is_lookup:
                 retval = FilterLookup[retval]
 
+    is_input = django_type.is_input
+    is_partial = django_type.is_partial
+    is_filter = bool(django_type.is_filter)
+
     if getattr(field, "primary_key", False):
-        # Primary keys are always required
+        # Primary keys are always required, unless this is a filter
         optional = False
     elif isinstance(field, ManyToManyField):
         # Many to many is always a list on get, but optional otherwise
-        optional = is_input or is_partial
+        optional = is_input or is_partial or is_filter
     elif isinstance(field, ForeignObjectRel):
-        optional = is_input or is_partial or field.null
+        optional = is_input or is_partial or is_filter or field.null
     else:
-        if is_partial:
+        if is_partial or is_filter:
             optional = True
         elif is_input:
             optional = field.blank or field.default is not NOT_PROVIDED
